@@ -58,11 +58,12 @@ function class:_init (options)
         self_.pageTemplate.firstContentFrame = self_.pageTemplate.frames[self_.firstContentFrame]
       end
       local frame = self_:initialFrame()
-      SILE.typesetter = SILE.defaultTypesetter(frame)
+      SILE.typesetter = SILE.typesetters.base(frame)
       SILE.typesetter:registerPageEndHook(function ()
-        if SU.debugging("frames") then
+        SU.debug("frames", function ()
           for _, v in pairs(SILE.frames) do SILE.outputter:debugFrame(v) end
-        end
+          return "Drew debug outlines around frames"
+        end)
       end)
     end)
 end
@@ -126,7 +127,7 @@ function class.declareSettings (_)
   })
   SILE.settings:declare({
     parameter = "current.hangIndent",
-    type = "integer or nil",
+    type = "measurement or nil",
     default = nil,
     help = "Size of hanging indent"
   })
@@ -187,7 +188,14 @@ end
 function class:registerHook (category, func)
   for _, func_ in ipairs(self.hooks[category]) do
     if func_ == func then
+      return
+      --[[ See https://github.com/sile-typesetter/sile/issues/1531
       return SU.warn("Attempted to set the same function hook twice, probably unintended, skipping.")
+      -- If the same function signature is already set a package is probably being
+      -- re-initialized. Ditch the first instance of the hook so that it runs in
+      -- the order of last initialization.
+      self.hooks[category][_] = nil
+      ]]
     end
   end
   table.insert(self.hooks[category], func)
@@ -195,7 +203,7 @@ end
 
 function class:runHooks (category, options)
   for _, func in ipairs(self.hooks[category]) do
-    SU.debug("classhooks", "Running hook from " .. category, options and "with options " .. #options)
+    SU.debug("classhooks", "Running hook from", category, options and "with options " .. #options)
     func(self, options)
   end
 end
@@ -409,7 +417,7 @@ function class:registerCommands ()
   end, "Declares (or re-declares) a frame on this page.")
 
   self:registerCommand("penalty", function (options, _)
-    if options.vertical and not SILE.typesetter:vmode() then
+    if SU.boolean(options.vertical, false) and not SILE.typesetter:vmode() then
       SILE.typesetter:leaveHmode()
     end
     if SILE.typesetter:vmode() then
@@ -422,19 +430,16 @@ function class:registerCommands ()
   self:registerCommand("discretionary", function (options, _)
     local discretionary = SILE.nodefactory.discretionary({})
     if options.prebreak then
-      SILE.call("hbox", {}, function () SILE.typesetter:typeset(options.prebreak) end)
-      discretionary.prebreak = { SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes] }
-      SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes] = nil
+      local hbox = SILE.typesetter:makeHbox({ options.prebreak })
+      discretionary.prebreak = { hbox }
     end
     if options.postbreak then
-      SILE.call("hbox", {}, function () SILE.typesetter:typeset(options.postbreak) end)
-      discretionary.postbreak = { SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes] }
-      SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes] = nil
+      local hbox = SILE.typesetter:makeHbox({ options.postbreak })
+      discretionary.postbreak = { hbox }
     end
     if options.replacement then
-      SILE.call("hbox", {}, function () SILE.typesetter:typeset(options.replacement) end)
-      discretionary.replacement = { SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes] }
-      SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes] = nil
+      local hbox = SILE.typesetter:makeHbox({ options.replacement })
+      discretionary.replacement = { hbox }
     end
     table.insert(SILE.typesetter.state.nodes, discretionary)
   end, "Inserts a discretionary node.")
@@ -450,7 +455,7 @@ function class:registerCommands ()
   end, "Inserts a glue node. The width option denotes the glue dimension.")
 
   self:registerCommand("skip", function (options, _)
-    options.discardable = options.discardable or false
+    options.discardable = SU.boolean(options.discardable, false)
     options.height = SILE.length(options.height):absolute()
     SILE.typesetter:leaveHmode()
     if options.discardable then
@@ -505,7 +510,19 @@ end
 
 -- WARNING: not called as class method
 function class.newPar (typesetter)
-  typesetter:pushGlue(SILE.settings:get("current.parindent") or SILE.settings:get("document.parindent"))
+  local parindent = SILE.settings:get("current.parindent") or SILE.settings:get("document.parindent")
+  -- See https://github.com/sile-typesetter/sile/issues/1361
+  -- The parindent *cannot* be pushed non-absolutized, as it may be evaluated
+  -- outside the (possibly temporary) setting scope where it was used for line
+  -- breaking.
+  -- Early absolutization can be problematic sometimes, but here we do not
+  -- really have the choice.
+  -- As of problematic cases, consider a parindent that would be defined in a
+  -- frame-related unit (%lw, %fw, etc.). If a frame break occurs and the next
+  -- frame has a different width, the parindent won't be re-evaluated in that
+  -- new frame context. However, defining a parindent in such a unit is quite
+  -- unlikely. And anyway pushback() has plenty of other issues.
+  typesetter:pushGlue(parindent:absolute())
   SILE.settings:set("current.parindent", nil)
   local hangIndent = SILE.settings:get("current.hangIndent")
   if hangIndent then
@@ -558,9 +575,9 @@ function class:finish ()
   end
   SILE.typesetter:runHooks("pageend") -- normally run by the typesetter
   self:endPage()
-    if SILE.typesetter then
-      assert(SILE.typesetter:isQueueEmpty(), "queues not empty")
-    end
+  if SILE.typesetter then
+    assert(SILE.typesetter:isQueueEmpty(), "queues not empty")
+  end
   SILE.outputter:finish()
   self:runHooks("finish")
 end
